@@ -1,6 +1,6 @@
-/* $Id: KeepAttributeElement.java,v 1.3.2.2 2007/01/18 21:31:51 eric Exp $
- *
- * ProGuard -- shrinking, optimization, and obfuscation of Java class files.
+/*
+ * ProGuard -- shrinking, optimization, obfuscation, and preverification
+ *             of Java bytecode.
  *
  * Copyright (c) 2002-2007 Eric Lafortune (eric@graphics.cornell.edu)
  *
@@ -20,9 +20,9 @@
  */
 package proguard.ant;
 
-import org.apache.tools.ant.types.*;
+import org.apache.tools.ant.types.DataType;
 
-import java.util.*;
+import java.util.List;
 
 /**
  * This DataType represents a named attribute in Ant.
