@@ -1,6 +1,6 @@
-/* $Id: ExtensionFileFilter.java,v 1.6.2.2 2007/01/18 21:31:52 eric Exp $
- *
- * ProGuard -- shrinking, optimization, and obfuscation of Java class files.
+/*
+ * ProGuard -- shrinking, optimization, obfuscation, and preverification
+ *             of Java bytecode.
  *
  * Copyright (c) 2002-2007 Eric Lafortune (eric@graphics.cornell.edu)
  *
@@ -20,8 +20,8 @@
  */
 package proguard.gui;
 
-import java.io.File;
 import javax.swing.filechooser.FileFilter;
+import java.io.File;
 
 
 /**
@@ -30,10 +30,10 @@ import javax.swing.filechooser.FileFilter;
  *
  * @author Eric Lafortune
  */
-class ExtensionFileFilter extends FileFilter
+final class ExtensionFileFilter extends FileFilter
 {
-    private String   description;
-    private String[] extensions;
+    private final String   description;
+    private final String[] extensions;
 
 
     /**
