@@ -1,6 +1,6 @@
-/* $Id: CompositeSprite.java,v 1.7.2.2 2007/01/18 21:31:52 eric Exp $
- *
- * ProGuard -- shrinking, optimization, and obfuscation of Java class files.
+/*
+ * ProGuard -- shrinking, optimization, obfuscation, and preverification
+ *             of Java bytecode.
  *
  * Copyright (c) 2002-2007 Eric Lafortune (eric@graphics.cornell.edu)
  *
@@ -20,7 +20,7 @@
  */
 package proguard.gui.splash;
 
-import java.awt.Graphics;
+import java.awt.*;
 
 /**
  * This Sprite is the composition of a list of Sprite objects.
@@ -29,7 +29,7 @@ import java.awt.Graphics;
  */
 public class CompositeSprite implements Sprite
 {
-    private Sprite[] sprites;
+    private final Sprite[] sprites;
 
 
     /**
