@@ -1,6 +1,6 @@
-/* $Id: OverrideGraphics2D.java,v 1.7.2.2 2007/01/18 21:31:52 eric Exp $
- *
- * ProGuard -- shrinking, optimization, and obfuscation of Java class files.
+/*
+ * ProGuard -- shrinking, optimization, obfuscation, and preverification
+ *             of Java bytecode.
  *
  * Copyright (c) 2002-2007 Eric Lafortune (eric@graphics.cornell.edu)
  *
@@ -35,10 +35,11 @@ import java.util.Map;
  * change those settings.
  *
  * @author Eric Lafortune
+ * @noinspection deprecation
  */
-class OverrideGraphics2D extends Graphics2D
+final class OverrideGraphics2D extends Graphics2D
 {
-    private Graphics2D graphics;
+    private final Graphics2D graphics;
 
     private Color  overrideColor;
     private Font   overrideFont;
