@@ -1,6 +1,6 @@
-/* $Id: SmoothTiming.java,v 1.7.2.2 2007/01/18 21:31:52 eric Exp $
- *
- * ProGuard -- shrinking, optimization, and obfuscation of Java class files.
+/*
+ * ProGuard -- shrinking, optimization, obfuscation, and preverification
+ *             of Java bytecode.
  *
  * Copyright (c) 2002-2007 Eric Lafortune (eric@graphics.cornell.edu)
  *
@@ -27,8 +27,8 @@ package proguard.gui.splash;
  */
 public class SmoothTiming implements Timing
 {
-    private long fromTime;
-    private long toTime;
+    private final long fromTime;
+    private final long toTime;
 
 
     /**
