@@ -1,6 +1,6 @@
-/* $Id: TextSprite.java,v 1.7.2.2 2007/01/18 21:31:52 eric Exp $
- *
- * ProGuard -- shrinking, optimization, and obfuscation of Java class files.
+/*
+ * ProGuard -- shrinking, optimization, obfuscation, and preverification
+ *             of Java bytecode.
  *
  * Copyright (c) 2002-2007 Eric Lafortune (eric@graphics.cornell.edu)
  *
@@ -29,29 +29,23 @@ import java.awt.*;
  */
 public class TextSprite implements Sprite
 {
-    private VariableString[] text;
-    private VariableInt      spacing;
-    private VariableFont     font;
-    private VariableColor    color;
-    private VariableInt      x;
-    private VariableInt      y;
+    private final VariableString[] text;
+    private final VariableInt      spacing;
+    private final VariableInt      x;
+    private final VariableInt      y;
 
 
     /**
      * Creates a new TextSprite containing a single line of text.
      * @param text  the variable text string.
-     * @param font  the variable text font.
-     * @param color the variable color.
      * @param x     the variable x-coordinate of the lower-left corner of the text.
      * @param y     the variable y-coordinate of the lower-left corner of the text.
      */
     public TextSprite(VariableString text,
-                      VariableFont   font,
-                      VariableColor  color,
                       VariableInt    x,
                       VariableInt    y)
     {
-        this(new VariableString[] { text }, new ConstantInt(0), font, color, x, y);
+        this(new VariableString[] { text }, new ConstantInt(0), x, y);
     }
 
 
@@ -59,8 +53,6 @@ public class TextSprite implements Sprite
      * Creates a new TextSprite containing a multiple lines of text.
      * @param text    the variable text strings.
      * @param spacing the variable spacing between the lines of text.
-     * @param font    the variable text font.
-     * @param color   the variable color.
      * @param x       the variable x-coordinate of the lower-left corner of the
      *                first line of text.
      * @param y       the variable y-coordinate of the lower-left corner of the
@@ -68,16 +60,12 @@ public class TextSprite implements Sprite
      */
     public TextSprite(VariableString[] text,
                       VariableInt      spacing,
-                      VariableFont     font,
-                      VariableColor    color,
                       VariableInt      x,
                       VariableInt      y)
     {
 
         this.text    = text;
         this.spacing = spacing;
-        this.font    = font;
-        this.color   = color;
         this.x       = x;
         this.y       = y;
     }
@@ -91,12 +79,11 @@ public class TextSprite implements Sprite
         int xt = x.getInt(time);
         int yt = y.getInt(time);
 
-        graphics.setFont(font.getFont(time));
-        graphics.setColor(color.getColor(time));
+        int spacingt = spacing.getInt(time);
 
         for (int index = 0; index < text.length; index++)
         {
-            graphics.drawString(text[index].getString(time), xt, yt + index * spacing.getInt(time));
+            graphics.drawString(text[index].getString(time), xt, yt + index * spacingt);
         }
     }
 }
