@@ -1,6 +1,6 @@
-/* $Id: TypeWriterString.java,v 1.7.2.2 2007/01/18 21:31:52 eric Exp $
- *
- * ProGuard -- shrinking, optimization, and obfuscation of Java class files.
+/*
+ * ProGuard -- shrinking, optimization, obfuscation, and preverification
+ *             of Java bytecode.
  *
  * Copyright (c) 2002-2007 Eric Lafortune (eric@graphics.cornell.edu)
  *
@@ -29,8 +29,8 @@ package proguard.gui.splash;
  */
 public class TypeWriterString implements VariableString
 {
-    private String string;
-    private Timing timing;
+    private final String string;
+    private final Timing timing;
 
     private int    cachedLength = -1;
     private String cachedString;
