@@ -1,6 +1,6 @@
-/* $Id: FileDataEntry.java,v 1.3.2.2 2007/01/18 21:31:52 eric Exp $
- *
- * ProGuard -- shrinking, optimization, and obfuscation of Java class files.
+/*
+ * ProGuard -- shrinking, optimization, obfuscation, and preverification
+ *             of Java bytecode.
  *
  * Copyright (c) 2002-2007 Eric Lafortune (eric@graphics.cornell.edu)
  *
@@ -31,8 +31,8 @@ import java.io.*;
  */
 public class FileDataEntry implements DataEntry
 {
-    private File        directory;
-    private File        file;
+    private final File        directory;
+    private final File        file;
     private InputStream inputStream;
 
 
