@@ -1,6 +1,6 @@
-/* $Id: JarReader.java,v 1.3.2.2 2007/01/18 21:31:52 eric Exp $
- *
- * ProGuard -- shrinking, optimization, and obfuscation of Java class files.
+/*
+ * ProGuard -- shrinking, optimization, obfuscation, and preverification
+ *             of Java bytecode.
  *
  * Copyright (c) 2002-2007 Eric Lafortune (eric@graphics.cornell.edu)
  *
@@ -20,7 +20,7 @@
  */
 package proguard.io;
 
-import java.io.*;
+import java.io.IOException;
 import java.util.zip.*;
 
 /**
@@ -31,7 +31,7 @@ import java.util.zip.*;
  */
 public class JarReader implements DataEntryReader
 {
-    DataEntryReader dataEntryReader;
+    private final DataEntryReader dataEntryReader;
 
 
     /**
