@@ -1,6 +1,6 @@
-/* $Id: MappingProcessor.java,v 1.5.2.2 2007/01/18 21:31:52 eric Exp $
- *
- * ProGuard -- shrinking, optimization, and obfuscation of Java class files.
+/*
+ * ProGuard -- shrinking, optimization, obfuscation, and preverification
+ *             of Java bytecode.
  *
  * Copyright (c) 2002-2007 Eric Lafortune (eric@graphics.cornell.edu)
  *
@@ -23,7 +23,7 @@ package proguard.obfuscate;
 
 /**
  * This interface specifies methods to process name mappings between original
- * class files and their obfuscated versions. The mappings are typically read
+ * classes and their obfuscated versions. The mappings are typically read
  * from a mapping file.
  *
  * @see MappingReader
@@ -40,8 +40,8 @@ public interface MappingProcessor
      * @return whether the processor is interested in receiving mappings of the
      *         class members of this class.
      */
-    public boolean processClassFileMapping(String className,
-                                           String newClassName);
+    public boolean processClassMapping(String className,
+                                       String newClassName);
 
     /**
      * Processes the given field name mapping.
